package edu.gustavus.mcs178.compounder;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Karl Knight in May 2011 to use Java Swing
 */
public class Compounder extends JFrame implements ItemListener {

	public static final int WINDOW_WIDTH = 600;
	public static final int WINDOW_HEIGHT = 800;

	private JTextArea outputArea;
	private CompoundingThread compThread;
	private JCheckBox runCheckbox;

	public Compounder(){
		super();
		
		setTitle("Compounder");
		setSize(WINDOW_WIDTH, WINDOW_HEIGHT);

		compThread = new CompoundingThread(this);

		JPanel controlPanel = new JPanel();
		add(controlPanel, BorderLayout.NORTH);

		outputArea = new JTextArea();
		outputArea.setEditable(false);
		add(outputArea, BorderLayout.CENTER);

		controlPanel.add(new JLabel("Initial amount:", JLabel.RIGHT));
		controlPanel.add(new InitialAmountField(1000.00, compThread));
		controlPanel.add(new JLabel("Interest rate:", JLabel.RIGHT));
		controlPanel.add(new InterestRateField(.05, compThread));
		runCheckbox = new JCheckBox("Run");
		runCheckbox.addItemListener(this);
		controlPanel.add(runCheckbox);

		compThread.start();
	}

	public void outputLine(String line){
		outputArea.append(line + "\n");
	}

	public void clearOutput(){
		outputArea.setText("");
	}

	public void stop(){
		compThread.disable();
	}

	public void start(){
		if(runCheckbox.isSelected()){
			compThread.enable();
		}
	}

	public void itemStateChanged(ItemEvent event) {
		if (event.getStateChange() == ItemEvent.SELECTED) {
			compThread.enable();
		} else {
			compThread.disable();
		}
	}
	
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				Compounder frame = 
					new Compounder();
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setVisible(true);
			}
		});
	}
}
