package edu.gustavus.mcs178.npuzzle.v4;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.GridLayout;
import java.awt.HeadlessException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Karl Knight in May 2011 to use Java Swing
 */
public class Puzzle extends JFrame {
	
	// The tiles of the puzzle are a square grid of PUZZLE_SIZE by PUZZLE_SIZE tiles
	public static final int PUZZLE_SIZE = 4;
	
	// Each tile is an approximate square of approximately this size in pixels
	public static final int BUTTON_WIDTH = 70;
	
	// Computed dimensions of the JFrame
	public static final int WINDOW_WIDTH = PUZZLE_SIZE * BUTTON_WIDTH;
	public static final int WINDOW_HEIGHT = (PUZZLE_SIZE + 1) * BUTTON_WIDTH;

	private JButton[][] tiles;
	private int blankRow;
	private int blankCol;
	
	public Puzzle() throws HeadlessException {
		
		int numButtons = PUZZLE_SIZE * PUZZLE_SIZE;
		setTitle((numButtons - 1) + " Puzzle");
		setSize(WINDOW_WIDTH, WINDOW_HEIGHT);

		// controlPanel contains high-level controls for the game
		JPanel controlPanel = new JPanel();
		add(controlPanel, BorderLayout.NORTH);
		
		JButton initializeButton = new JButton("Initialize");
		initializeButton.addActionListener(new InitializeActionListener(this));
		controlPanel.add(initializeButton);
		
		// mainPanel contains the grid of tiles (which are JButtons)
		JPanel mainPanel = new JPanel(new GridLayout(PUZZLE_SIZE, PUZZLE_SIZE));
		tiles = new JButton[PUZZLE_SIZE][PUZZLE_SIZE];
		add(mainPanel, BorderLayout.CENTER);
		
		// set up the grid of tiles in mainPanel
		for (int row = 0; row < PUZZLE_SIZE; row++) {
			for (int col = 0; col < PUZZLE_SIZE; col++) {
				JButton tile = new JButton();
				tiles[row][col] = tile;
				tile.addActionListener(new TileActionListener(this, row, col));
				mainPanel.add(tile);
			}
		}
		initializeTiles();
	}
	
	public void initializeTiles(){
		int tileCount = 0;
		for(int row = 0; row < PUZZLE_SIZE; row++){
			for(int col = 0; col < PUZZLE_SIZE; col++){
				tileCount++;
				tiles[row][col].setText("" + tileCount);
			}
		}
		blankRow = PUZZLE_SIZE - 1;
		blankCol = PUZZLE_SIZE - 1;
		tiles[blankRow][blankCol].setText("");
	}

	public void pushTile(int row, int col){
		if (row == blankRow){
			for ( ; blankCol < col; blankCol++) {
				tiles[blankRow][blankCol].setText
				(tiles[blankRow][blankCol+1].getText());
			}
			for ( ; blankCol > col; blankCol--) {
				tiles[blankRow][blankCol].setText
				(tiles[blankRow][blankCol-1].getText());
			}
		} else if (col == blankCol){
			for ( ; blankRow < row; blankRow++) {
				tiles[blankRow][blankCol].setText
				(tiles[blankRow+1][blankCol].getText());
			}
			for ( ; blankRow > row; blankRow--) {
				tiles[blankRow][blankCol].setText
				(tiles[blankRow-1][blankCol].getText());
			}
		}
		tiles[blankRow][blankCol].setText("");
	}
	
	public void pushRandomTile() {
		int row = (int) (Math.random() * PUZZLE_SIZE);
		int col = (int) (Math.random() * PUZZLE_SIZE);
		pushTile(row, col);
	}
	
	public void randomizeTiles() {
		for (int i=0; i < 100; i++) {
			pushRandomTile();
		}
	}
	
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable()
		{
			public void run() {
				Puzzle frame = 
					new Puzzle();
				frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
				frame.setVisible(true);
			}
		});
	}
}
